; Bouncing Balls in Bouncing Sprite Box - 11-Mar-2025 by Kevin Krausnick
; Assembler: Acme

* = $c000         ; sys 49152

; ------------------------------------------------------------------------------

init
  lda #$93        ; clear screen
  jsr $ffd2
  lda #10         ; light red
  sta $d028       ; sprite 1 color
  ldx #128
  stx 2040        ; sprite 0 data pointer
  inx
  stx 2041        ; sprite 1 data pointer
  inx
  stx 2044        ; sprite 4 data pointer
  inx
  stx 2046        ; sprite 6 data pointer
  lda #24
  sta $d000       ; box x pos
  lda #50
  sta $d001       ; box y pos
  lda #$53
  sta $d017       ; expand y
  sta $d01d       ; expand x
  sta $d015       ; enable sprites
  jsr render_empty_box

; ------------------------------------------------------------------------------

main
  jsr animate_balls
  clc
  lda #2          ; time delay
  adc $a2
- cmp $a2
  bne -
m1
  lda $d012       ; wait for vertical retrace
  bne m1
  lda box_dir_x
  beq +
  jsr move_box_right
  jmp m2
+ jsr move_box_left
m2
  lda box_dir_y
  beq +
  jsr move_box_down
  jmp m3
+ jsr move_box_up
m3
  lda $d000       ; mirror sprite x loc
  sta $d002
  sta $d008
  sta $d00c
  lda $d001       ; mirror sprite y loc
  sta $d003
  sta $d009
  sta $d00d
  bne main

; ------------------------------------------------------------------------------

render_empty_box  ; render empty box sprite
  ldy #0          ; clear sprites
  tya
  sta $d021       ; set background color to black
- sta 128*64,y
  iny
  bne -
  lda #$ff        ; sprite top and bottom borders
  ldx #2
- sta 128*64,x
  sta 128*64+3*20,x
  dex
  bpl -
- inx             ; sprite left and right borders
  lda #$80
  sta 128*64+3,x
  inx
  inx
  lda #$01
  sta 128*64+3,x
  cpx #56
  bne -
  rts

; ------------------------------------------------------------------------------

move_box_right
  inc $d000       ; move right one pixel
  bne +           ; boundary crossed?
  lda $d010       ; yes, so flip msb...
  eor #$53
  sta $d010
  bne mr_end      ; ...and exit routine
+ lda $d010       ; check msb
  beq mr_end      ; exit if sprite is on left side of screen
  lda $d000       ; sprite is on right side of screen, so...
  cmp #40         ; ...check for screen boundary
  bcc mr_end
  dec box_dir_x   ; reverse box x direction
mr_end
  rts

move_box_left
  dec $d000       ; move left one pixel
  lda $d000
  cmp #$ff
  bne +           ; boundary crossed?
  lda $d010       ; yes, so flip msb...
  eor #$53
  sta $d010
  beq ml_end      ; ...and exit routine
+ lda $d010       ; check msb
  bne ml_end      ; exit if sprite is on right side of screen
  lda $d000       ; sprite is on left side of screen, so...
  cmp #25         ; ...check for screen boundary
  bcs ml_end
  inc box_dir_x   ; reverse box x direction
ml_end
  rts

move_box_down
  inc $d001       ; move down one pixel
  lda $d001
  cmp #208
  bcc +
  dec box_dir_y   ; reverse box y direction
+ rts

move_box_up
  dec $d001       ; move up one pixel
  lda $d001
  cmp #51
  bcs +
  inc box_dir_y   ; reverse box y direction
+ rts

; ------------------------------------------------------------------------------

animate_balls
  lda as2+1
  pha
  pha
  jsr as1         ; animate ball 1
  pla
  clc
  adc #8
  sta as2+1
  sta as3+1
  pha
  jsr as1         ; animate ball 2
  pla
  clc
  adc #8
  sta as2+1
  sta as3+1
  jsr as1         ; animate ball 3
  pla
  sta as2+1
  sta as3+1
  rts

as1
  ldy #7
as2
- lda ball1_params,y
  sta ball_params,y
  dey
  bpl -
  jsr animate_ball
  ldy #7
- lda ball_params,y
as3
  sta ball1_params,y
  dey
  bpl -
  rts

animate_ball
  lda #0          ; clear ball x row
  ldx ball_offset
  sta $2040,x
  sta $2041,x
  sta $2042,x
  lda ball_dir_x
  beq ab1
  lsr ball_x      ; move ball right
  ror ball_x+1
  ror ball_x+2
  lda ball_x+2
  cmp #2
  bne +
  dec ball_dir_x
+ jmp ab2
ab1
  asl ball_x+2    ; move ball left
  rol ball_x+1
  rol ball_x
  lda ball_x
  cmp #64
  bne ab2
  inc ball_dir_x
ab2
  lda ball_dir_y
  beq ab3
  lda ball_offset ; move ball down
  clc
  adc #3
  sta ball_offset
  cmp ball_range
  bcc +
  dec ball_dir_y
+ jmp ab4
ab3
  lda ball_offset ; move ball up
  sec
  sbc #3
  sta ball_offset
  cmp ball_range+1
  bcs ab4
  inc ball_dir_y
ab4
  ldx ball_offset ; render ball
  lda ball_x
  sta $2040,x
  lda ball_x+1
  sta $2041,x
  lda ball_x+2
  sta $2042,x
  rts

; ------------------------------------------------------------------------------

box_dir_x    !byte 1 ; box x direction (0 = left, 1 = right)
box_dir_y    !byte 1 ; box y direction (0 = up, 1 = down)

ball1_params !byte 1,1,3,57,6,64,0,0
ball2_params !byte 1,1,94,121,70,0,0,4
ball3_params !byte 1,0,149,185,134,0,8,0

ball_params                  ; variables below are external to compiled program
ball_dir_x   = ball_params   ; ball x direction (0 = left, 1 = right)
ball_dir_y   = ball_params+1 ; ball y direction (0 = up, 1 = down)
ball_offset  = ball_params+2 ; ball base address offset
ball_range   = ball_params+3 ; upper and lower deflection ranges
ball_x       = ball_params+5 ; ball x row